package com.io.yutian.elementoriginlib;

import com.io.yutian.elementoriginlib.lang.Lang;
import com.io.yutian.elementoriginlib.listener.GuiHandlerListener;
import com.io.yutian.elementoriginlib.listener.PlayerChatInputListener;
import com.io.yutian.elementoriginlib.logger.Logger;
import com.io.yutian.elementoriginlib.manager.CommandManager;
import com.io.yutian.elementoriginlib.redis.RedisIO;
import net.byteflux.libby.*;
import net.byteflux.libby.logging.LogLevel;
import org.bukkit.plugin.java.JavaPlugin;
import org.slf4j.LoggerFactory;

public final class ElementOriginLib extends JavaPlugin {

    private static final org.slf4j.Logger log = LoggerFactory.getLogger(ElementOriginLib.class);
    private static ElementOriginLib instance;

    private static Logger logger = Logger.getLogger(ElementOriginLib.class);

    private RedisIO redisIO;

    @Override
    public void onEnable() {
        instance = this;

//        loadLibraries();

        new GuiHandlerListener(this);
        new PlayerChatInputListener(this);

        CommandManager.INSTANCE.registerPluginCommand("elementoriginlib");

        reload();

        redisIO = new RedisIO();
        redisIO.init(this);

        logger.info("Successfully load ElementOriginLib v"+getPluginMeta().getVersion());

    }

    private void loadLibraries() {
        logger.info("Loading libraries...");
        LibraryManager libraryManager = new BukkitLibraryManager(this);
        libraryManager.addMavenCentral();
        libraryManager.setLogLevel(LogLevel.WARN);
        libraryManager.loadLibrary(Library.builder().groupId("org{}apache{}commons").artifactId("commons-compress").version("1.27.1").build());
        libraryManager.loadLibrary(Library.builder().groupId("com{}github{}luben").artifactId("zstd-jni").version("1.5.6-9").build());
        libraryManager.loadLibrary(Library.builder().groupId("com{}fasterxml{}jackson{}core").artifactId("jackson-databind").version("2.18.2").build());
//        libraryManager.loadLibrary(Library.builder().groupId("com{}zaxxer").artifactId("HikariCP").version("6.2.1").build());
//        libraryManager.loadLibrary(Library.builder().groupId("org{}xerial").artifactId("sqlite-jdbc").version("3.46.0.0").build());
        libraryManager.loadLibrary(Library.builder().groupId("redis{}clients").id("jedis").artifactId("jedis").version("5.2.0").build());
        logger.info("Successfully loaded libraries.");
    }

    @Override
    public void onDisable() {
    }

    public void reload() {
        Lang.registerLangFile(this);
        Lang.reload();
    }

    public RedisIO getRedisIO() {
        return redisIO;
    }

    public static ElementOriginLib inst() {
        return instance;
    }

}
