package com.io.yutian.elementoriginlib.command;

import com.io.yutian.elementoriginlib.command.handler.CommandHandler;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandMap;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SimpleCommandManager implements ICommandManager {

    protected static Map<String, Command> bukkitCommandMap = new HashMap<>();
    protected static CommandMap commandMap;

    private final String name;

    @NotNull
    private Map<Class<?>, Object> commands = new HashMap<>();
    @NotNull
    private List<CommandEntity> commandEntities = new ArrayList<>();


    public SimpleCommandManager(@NotNull String name) {
        this.name = name;
    }

    public <T> void register(@NotNull Object instance) {
        if (instance == null) {
            throw new NullPointerException("instance is null");
        }
        Class<?> instanceClass = instance.getClass();
        CommandEntity command = CommandEntity.parseFromClass(instance, instanceClass);
        if (command == null) {
            return;
        }
        commands.put(instanceClass, instance);
        commandEntities.add(command);
    }

    public void unregisterAll() {
        commands.clear();
        commandEntities.clear();
    }

    public void unregister(Class<?> commandClass) {
        if (commandClass == null) {
            return;
        }
        commands.remove(commandClass);
        commandEntities.clear();
    }

    public void registerPluginCommand(@NotNull String commandName) {
        Bukkit.getPluginCommand(commandName).setExecutor(new CommandHandler(this));
    }

    @NotNull
    @Override
    public String getName() {
        return name;
    }

    @NotNull
    @Override
    public List<CommandEntity> getCommandEntities() {
        return commandEntities;
    }

    static {
        try {
            Class<?> c = Bukkit.getServer().getClass();
            for (Method method : c.getDeclaredMethods()) {
                if (method.getName().equals("getCommandMap")) {
                    commandMap = (CommandMap) method.invoke(Bukkit.getServer(), new Object[0]);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
