package com.io.yutian.elementoriginlib.item;

import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagCompound;
import net.kyori.adventure.text.Component;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

public class TagStatItemStackBuilder {

    private final TagStatItem tagStatItem;

    protected ItemStack itemStack;
    protected ItemMeta itemMeta;

    protected ItemProxy itemProxy;

    public TagStatItemStackBuilder(TagStatItem tagStatItem) {
        this.tagStatItem = tagStatItem;
        this.itemStack = tagStatItem.getItemStack();
        this.itemMeta = itemStack.getItemMeta();
        this.itemProxy = tagStatItem.getItemProxy();
    }

    public void editTag(Consumer<TagCompound> tagCompoundConsumer) {
        itemProxy.editTag(tagCompoundConsumer);
    }

    @NotNull
    public ItemStack build() {
        buildCompounds();
        return itemStack;
    }

    @NotNull
    public ItemMeta getItemMeta() {
        return itemMeta;
    }

    private void buildCompounds() {
        tagStatItem.getStats().forEach((stat, statData) -> stat.whenApplied(this, statData));
        itemStack = itemProxy.getItemStack();
        itemMeta = itemStack.getItemMeta();
        tagStatItem.getStats().forEach((stat, statData) -> stat.applyMeta(this, statData));
        List<Component> lores = itemMeta.hasLore() ? itemMeta.lore() : new ArrayList<>();
        tagStatItem.getStats().forEach((stat, statData) -> stat.whenApplyLore(this, statData, lores));
        itemMeta.lore(lores);
        itemStack.setItemMeta(itemMeta);
    }

    @NotNull
    public ItemProxy getItemProxy() {
        return itemProxy;
    }

    public TagStatItem getOriginItem() {
        return tagStatItem;
    }

}
