package com.io.yutian.elementoriginlib.item.stat;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItem;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

public abstract class ItemStat<S extends StatData> {

    @NotNull
    private final String id;
    @NotNull
    private final String tagPath;

    public ItemStat(@NotNull String id, @NotNull String path) {
        this.id = id;
        this.tagPath = path;
    }

    public abstract void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull S statData);

    public abstract S loadAsObject(@NotNull Object value) throws ItemStatDataLoadException;

    public void applyMeta(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull S statData) {
    }

    public void whenApplyLore(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull S statData, @NotNull List<String> lores) {
    }

    @Nullable
    public abstract S getLoadedTag(@NotNull ItemProxy itemProxy);

    public void load(@NotNull TagStatItem tagStatItem) {
        if (!tagStatItem.getItemProxy().has(getTagPath())) {
            return;
        }
        S loadedTag = getLoadedTag(tagStatItem.getItemProxy());
        if (loadedTag != null) {
            tagStatItem.setStatData(this, loadedTag);
        }
    }

    @NotNull
    public String getId() {
        return id;
    }

    @NotNull
    public String getTagPath() {
        return tagPath;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        ItemStat<?> itemStat = (ItemStat<?>) o;
        return id.equals(itemStat.id);
    }

    @Override
    public int hashCode() {
        return id.hashCode();
    }
}
