package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.BooleanData;
import com.io.yutian.elementoriginlib.tag.TagByte;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public abstract class BooleanStat extends ItemStat<BooleanData> {

    public BooleanStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull BooleanData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        itemProxy.editTag(tagCompound1 -> tagCompound1.putByte(getTagPath(), (byte) (statData.getValue() ? 1 : 0)));
    }

    @Override
    public BooleanData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Boolean bool) {
                return new BooleanData(bool);
            } else if (value instanceof String arg) {
                boolean parsed = Boolean.parseBoolean(arg);
                if (!arg.equalsIgnoreCase("true") && !arg.equalsIgnoreCase("false")) {
                    throw new ItemStatDataLoadException("Invalid boolean string: " + arg);
                }
                return new BooleanData(parsed);
            } else if (value instanceof Byte b) {
                return new BooleanData(b == 1);
            } else if (value instanceof Integer i) {
                return new BooleanData(i == 1);
            } else {
                throw new ItemStatDataLoadException("Invalid boolean value: " + value);
            }
        } catch (ClassCastException e) {
            throw new ItemStatDataLoadException("Invalid type: " + value.getClass(), e);
        }
    }

    @Nullable
    @Override
    public BooleanData getLoadedTag(@NotNull ItemProxy itemProxy) {
        return new BooleanData(((TagByte) itemProxy.get(getTagPath())).getByte() == 1);
    }

}
