package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.DoubleListData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagDouble;
import com.io.yutian.elementoriginlib.tag.TagList;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

public abstract class DoubleListStat extends ItemStat<DoubleListData> {

    public DoubleListStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull DoubleListData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        TagList<TagDouble> tagList = new TagList<>();
        statData.getList().forEach(s -> tagList.add(new TagDouble(s)));
        itemProxy.editTag(tagCompound1 -> tagCompound1.put(getTagPath(), tagList));
    }

    @Override
    public DoubleListData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<Double> doubleList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof Double d) {
                        doubleList.add(d);
                    } else if (item instanceof String str) {
                        // 如果是 String，尝试将其解析为 Double
                        try {
                            doubleList.add(Double.parseDouble(str));
                        } catch (NumberFormatException e) {
                            throw new ItemStatDataLoadException("Invalid string value for Double: " + str, e);
                        }
                    } else {
                        throw new ItemStatDataLoadException("List contains non-double element: " + item);
                    }
                }
                return new DoubleListData(doubleList);
            } else if (value instanceof double[] array) {
                List<Double> doubleList = new ArrayList<>();
                for (double d : array) {
                    doubleList.add(d);
                }
                return new DoubleListData(doubleList);
            } else if (value instanceof Double[] array) {
                List<Double> doubleList = new ArrayList<>(Arrays.asList(array));
                return new DoubleListData(doubleList);
            } else if (value instanceof String str) {
                List<Double> doubleList = new ArrayList<>();
                String[] items = str.split(",");
                for (String item : items) {
                    try {
                        doubleList.add(Double.parseDouble(item.trim()));
                    } catch (NumberFormatException e) {
                        throw new ItemStatDataLoadException("Invalid number format in string: " + item, e);
                    }
                }
                return new DoubleListData(doubleList);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for DoubleListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }


    @Nullable
    @Override
    public DoubleListData getLoadedTag(@NotNull ItemProxy itemProxy) {
        if (!itemProxy.has(getTagPath(), TagList.TYPE_ID)) {
            return new DoubleListData(new ArrayList<>());
        }
        TagList<TagDouble> tagList = (TagList<TagDouble>) itemProxy.get(getTagPath());
        List<Double> list = new ArrayList<>();
        for (int i = 0; i < tagList.size(); i++) {
            list.add(tagList.get(i).getDouble());
        }
        return new DoubleListData(list);
    }

}
