package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.DoubleData;
import com.io.yutian.elementoriginlib.tag.TagDouble;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public abstract class DoubleStat extends ItemStat<DoubleData> {

    public DoubleStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull DoubleData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        itemProxy.editTag(tagCompound1 -> tagCompound1.putDouble(getTagPath(), statData.getValue()));
    }

    @Override
    public DoubleData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Double d) {
                return new DoubleData(d);
            } else if (value instanceof String arg) {
                try {
                    return new DoubleData(Double.parseDouble(arg));
                } catch (NumberFormatException e) {
                    throw new ItemStatDataLoadException("Invalid double string: " + arg, e);
                }
            } else if (value instanceof Byte b) {
                return new DoubleData((double) b);
            } else if (value instanceof Integer i) {
                return new DoubleData((double) i);
            } else if (value instanceof Long l) {
                return new DoubleData((double) l);
            } else {
                throw new ItemStatDataLoadException("Invalid double value: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    @Nullable
    @Override
    public DoubleData getLoadedTag(@NotNull ItemProxy itemProxy) {
        return new DoubleData(((TagDouble) itemProxy.get(getTagPath())).getDouble());
    }

}
