package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.EnumListData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagList;
import com.io.yutian.elementoriginlib.tag.TagString;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;


public abstract class EnumListStat<E extends Enum> extends ItemStat<EnumListData<E>> {

    private Class<E> clazz;

    public EnumListStat(@NotNull String id, @NotNull String path, Class<E> clazz) {
        super(id, path);
        this.clazz = clazz;
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull EnumListData<E> statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        TagList<TagString> tagList = new TagList<>();
        statData.getList().forEach(s -> tagList.add(new TagString(s.name())));
        itemProxy.editTag(tagCompound1 -> tagCompound1.put(getTagPath(), tagList));
    }

    @Override
    public EnumListData<E> loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<E> enumList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof String str) {
                        try {
                            E enumConstant = (E) Enum.valueOf(clazz, str);
                            enumList.add(enumConstant);
                        } catch (IllegalArgumentException e) {
                            throw new ItemStatDataLoadException("Invalid enum string: " + str, e);
                        }
                    } else if (item instanceof Integer i) {
                        E[] enumConstants = clazz.getEnumConstants();
                        if (i < 0 || i >= enumConstants.length) {
                            throw new ItemStatDataLoadException("Invalid enum ordinal: " + i);
                        }
                        enumList.add(enumConstants[i]);
                    } else if (item instanceof Enum<?> enumItem) {
                        enumList.add((E) enumItem);
                    } else {
                        throw new ItemStatDataLoadException("Invalid list element type: " + item);
                    }
                }
                return new EnumListData<>(enumList);
            } else if (value instanceof String str) {
                List<E> enumList = new ArrayList<>();
                String[] enumNames = str.split(",");
                for (String enumName : enumNames) {
                    try {
                        E enumConstant = (E) Enum.valueOf(clazz, enumName.trim());
                        enumList.add(enumConstant);
                    } catch (IllegalArgumentException e) {
                        throw new ItemStatDataLoadException("Invalid enum string: " + enumName, e);
                    }
                }
                return new EnumListData<>(enumList);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for EnumListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    @Override
    public @Nullable EnumListData<E> getLoadedTag(@NotNull ItemProxy itemProxy) {
        if (!itemProxy.has(getTagPath(), TagList.TYPE_ID)) {
            return new EnumListData<>(new ArrayList<>());
        }
        TagList<TagString> tagList = (TagList<TagString>) itemProxy.get(getTagPath());
        List<Enum> list = new ArrayList<>();
        for (int i = 0; i < tagList.size(); i++) {
            list.add(Enum.valueOf(clazz, tagList.get(i).getString()));
        }
        return new EnumListData(list);
    }

}
