package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.EnumData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagString;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public abstract class EnumStat<E extends Enum> extends ItemStat<EnumData<E>> {

    private Class<E> clazz;

    public EnumStat(@NotNull String id, @NotNull String path, Class<E> clazz) {
        super(id, path);
        this.clazz = clazz;
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull EnumData<E> statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        itemProxy.editTag(tagCompound1 -> tagCompound1.putString(getTagPath(), statData.getValue().name()));
    }

    @Override
    public EnumData<E> loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Enum<?> enumValue) {
                return new EnumData<>((E) enumValue);
            } else if (value instanceof String str) {
                try {
                    E enumConstant = (E) Enum.valueOf(clazz, str);
                    return new EnumData<>(enumConstant);
                } catch (IllegalArgumentException e) {
                    throw new ItemStatDataLoadException("Invalid enum string: " + str, e);
                }
            } else if (value instanceof Integer i) {
                E[] enumConstants = clazz.getEnumConstants();
                if (i < 0 || i >= enumConstants.length) {
                    throw new ItemStatDataLoadException("Invalid enum ordinal: " + i);
                }
                return new EnumData<>(enumConstants[i]);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for EnumData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    @Override
    public @Nullable EnumData<E> getLoadedTag(@NotNull ItemProxy itemProxy) {
        return new EnumData(Enum.valueOf(clazz, ((TagString) itemProxy.get(getTagPath())).getString()));
    }

}
