package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.IntListData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagInt;
import com.io.yutian.elementoriginlib.tag.TagList;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public abstract class IntListStat extends ItemStat<IntListData> {

    public IntListStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull IntListData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        TagList<TagInt> tagList = new TagList<>();
        statData.getList().forEach(s -> tagList.add(new TagInt(s)));
        itemProxy.editTag(tagCompound1 -> tagCompound1.put(getTagPath(), tagList));
    }

    @Override
    public IntListData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<Integer> intList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof Integer i) {
                        intList.add(i);
                    } else {
                        throw new ItemStatDataLoadException("List contains non-integer element: " + item);
                    }
                }
                return new IntListData(intList);
            } else if (value instanceof int[] array) {
                List<Integer> intList = new ArrayList<>();
                for (int i : array) {
                    intList.add(i);
                }
                return new IntListData(intList);
            } else if (value instanceof String str) {
                try {
                    List<Integer> intList = new ArrayList<>();
                    String[] items = str.split(",");
                    for (String item : items) {
                        intList.add(Integer.parseInt(item.trim()));
                    }
                    return new IntListData(intList);
                } catch (NumberFormatException e) {
                    throw new ItemStatDataLoadException("Invalid integer string: " + str, e);
                }
            } else {
                throw new ItemStatDataLoadException("Invalid value type for IntListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    @Nullable
    @Override
    public IntListData getLoadedTag(@NotNull ItemProxy itemProxy) {
        if (!itemProxy.has(getTagPath(), TagList.TYPE_ID)) {
            return new IntListData(new ArrayList<>());
        }
        TagList<TagInt> tagList = (TagList<TagInt>) itemProxy.get(getTagPath());
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < tagList.size(); i++) {
            list.add(tagList.get(i).getInt());
        }
        return new IntListData(list);
    }

}
