package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.IntData;
import com.io.yutian.elementoriginlib.tag.TagInt;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public abstract class IntStat extends ItemStat<IntData> {

    public IntStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull IntData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        itemProxy.editTag(tagCompound1 -> tagCompound1.putInt(getTagPath(), statData.getInt()));
    }

    @Override
    public IntData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Integer i) {
                return new IntData(i);
            } else if (value instanceof String arg) {
                try {
                    return new IntData(Integer.parseInt(arg));
                } catch (NumberFormatException e) {
                    throw new ItemStatDataLoadException("Invalid integer string: " + arg, e);
                }
            } else if (value instanceof Byte b) {
                return new IntData((int) b);
            } else if (value instanceof Long l) {
                if (l < Integer.MIN_VALUE || l > Integer.MAX_VALUE) {
                    throw new ItemStatDataLoadException("Long value out of range for integer: " + l);
                }
                return new IntData(l.intValue());
            } else if (value instanceof Double d) {
                if (d < Integer.MIN_VALUE || d > Integer.MAX_VALUE) {
                    throw new ItemStatDataLoadException("Double value out of range for integer: " + d);
                }
                return new IntData((int) d.doubleValue());
            } else if (value instanceof Float f) {
                if (f < Integer.MIN_VALUE || f > Integer.MAX_VALUE) {
                    throw new ItemStatDataLoadException("Float value out of range for integer: " + f);
                }
                return new IntData((int) f.floatValue());
            } else {
                throw new ItemStatDataLoadException("Invalid integer value: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }




    @Nullable
    @Override
    public IntData getLoadedTag(@NotNull ItemProxy itemProxy) {
        return new IntData(((TagInt) itemProxy.get(getTagPath())).getInt());
    }

}
