package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.MapData;
import com.io.yutian.elementoriginlib.tag.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class MapStat extends ItemStat<MapData> {

    public MapStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull MapData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        TagCompound tagCompound = new TagCompound();
        for (Map.Entry<String, Object> entry : statData.getMap().entrySet()) {
            tagCompound.put(entry.getKey(), ITag.asObject(entry.getValue()));
        }
        itemProxy.editTag(tagCompound1 -> tagCompound1.put(getTagPath(), tagCompound));
    }

    @Override
    public MapData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof Map<?, ?> map) {
                Map<String, Object> resultMap = new HashMap<>();
                for (Map.Entry<?, ?> entry : map.entrySet()) {
                    if (entry.getKey() instanceof String key) {
                        resultMap.put(key, entry.getValue());
                    } else {
                        throw new ItemStatDataLoadException("Map key is not a String: " + entry.getKey());
                    }
                }
                return new MapData(resultMap);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for MapData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    @Override
    public @Nullable MapData getLoadedTag(@NotNull ItemProxy itemProxy) {
        if (!itemProxy.has(getTagPath(), TagCompound.TYPE_ID)) {
            return new MapData(new HashMap<>());
        }
        TagCompound tagCompound = (TagCompound) itemProxy.get(getTagPath());
        Map<String, Object> map = new HashMap<>();
        for (Map.Entry<String, ITag> entry : tagCompound.getMap().entrySet()) {
            map.put(entry.getKey(), entry.getValue().getValue());
        }
        return new MapData(map);
    }

}
