package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.StringListData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagList;
import com.io.yutian.elementoriginlib.tag.TagString;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

public abstract class StringListStat extends ItemStat<StringListData> {

    public StringListStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull StringListData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        TagList<TagString> tagList = new TagList<>();
        statData.getList().forEach(s -> tagList.add(new TagString(s)));
        itemProxy.editTag(tagCompound1 -> tagCompound1.put(getTagPath(), tagList));
    }

    @Override
    public StringListData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof List<?> list) {
                List<String> stringList = new ArrayList<>();
                for (Object item : list) {
                    if (item instanceof String str) {
                        stringList.add(str);
                    } else {
                        throw new ItemStatDataLoadException("List contains non-string element: " + item);
                    }
                }
                return new StringListData(stringList);
            } else if (value instanceof String[] array) {
                // 处理 String[] 类型
                return new StringListData(Arrays.asList(array));
            } else if (value instanceof String str) {
                List<String> stringList = new ArrayList<>();
                stringList.add(str);
                return new StringListData(stringList);
            } else {
                throw new ItemStatDataLoadException("Invalid value type for StringListData: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }


    @Nullable
    @Override
    public StringListData getLoadedTag(@NotNull ItemProxy itemProxy) {
        if (!itemProxy.has(getTagPath(), TagList.TYPE_ID)) {
            return new StringListData(new ArrayList<>());
        }
        TagList<TagString> tagList = (TagList<TagString>) itemProxy.get(getTagPath());
        List<String> list = new ArrayList<>();
        for (int i = 0; i < tagList.size(); i++) {
            list.add(tagList.get(i).getString());
        }
        return new StringListData(list);
    }

}
