package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.exception.itemstat.ItemStatDataLoadException;
import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.UUIDData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagString;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;
import java.util.UUID;

public abstract class UUIDStat extends ItemStat<UUIDData> {

    public UUIDStat(@NotNull String id, @NotNull String path) {
        super(id, path);
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull UUIDData statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        itemProxy.editTag(tagCompound1 -> tagCompound1.putString(getTagPath(), statData.getUUID().toString()));
    }

    @Override
    public UUIDData loadAsObject(@NotNull Object value) throws ItemStatDataLoadException {
        if (Objects.isNull(value)) {
            throw new ItemStatDataLoadException("Invalid value: null");
        }
        try {
            if (value instanceof UUID uuid) {
                return new UUIDData(uuid);
            } else if (value instanceof String str) {
                try {
                    return new UUIDData(UUID.fromString(str));
                } catch (IllegalArgumentException e) {
                    throw new ItemStatDataLoadException("Invalid UUID string: " + str, e);
                }
            } else {
                throw new ItemStatDataLoadException("Invalid UUID value: " + value);
            }
        } catch (Exception e) {
            throw new ItemStatDataLoadException(e);
        }
    }

    @Nullable
    @Override
    public UUIDData getLoadedTag(@NotNull ItemProxy itemProxy) {
        return new UUIDData(UUID.fromString(((TagString) itemProxy.get(getTagPath())).getString()));
    }

}
