package com.io.yutian.elementoriginlib.list;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.IntStream;

public class PageList<T> {

	private Map<Integer, List<T>> map = new HashMap<>();
	private final int totalPages;

	public PageList(List<T> list, int amount) {
		if (list == null || list.isEmpty() || amount <= 0) {
			totalPages = 0;
		} else {
			totalPages = (list.size() - 1) / amount + 1;
			IntStream.rangeClosed(1, totalPages).forEach(page -> {
				int start = (page - 1) * amount;
				int end = Math.min(page * amount, list.size());
				map.put(page, new ArrayList<>(list.subList(start, end)));
			});
		}
	}

	public int size() {
		return totalPages;
	}

	public List<T> getList(int page) {
		if (page < 1 || page > totalPages) {
			throw new IllegalArgumentException("Invalid page number: " + page);
		}
		return map.get(page);
	}

	public boolean hasNextPage(int page) {
		return page < totalPages;
	}

	public int getTotalPages() {
		return totalPages;
	}
}
