package com.io.yutian.elementoriginlib.serialize;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.Module;
import com.fasterxml.jackson.databind.introspect.VisibilityChecker;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.module.afterburner.AfterburnerModule;
import com.io.yutian.elementoriginlib.logger.Logger;
import com.io.yutian.elementoriginlib.serialize.serializers.*;
import org.bukkit.inventory.ItemStack;

import java.util.UUID;

public class SerializeHelper {

    private static final Logger LOGGER = Logger.getLogger(SerializeHelper.class);

    private static final ObjectMapper objectMapper;
    private static ObjectWriter objectWriter;
    private static ObjectReader objectReader;

    static {
        objectMapper = new ObjectMapper();
        objectMapper.setVisibility(
                VisibilityChecker.Std.defaultInstance()
                        .withFieldVisibility(JsonAutoDetect.Visibility.ANY)
                        .withGetterVisibility(JsonAutoDetect.Visibility.NONE)
                        .withSetterVisibility(JsonAutoDetect.Visibility.NONE)
                        .withIsGetterVisibility(JsonAutoDetect.Visibility.NONE)
                        .withCreatorVisibility(JsonAutoDetect.Visibility.NONE)
        );

        objectMapper.registerModule(new AfterburnerModule());

        objectMapper.disable(MapperFeature.AUTO_DETECT_GETTERS);
        objectMapper.disable(MapperFeature.AUTO_DETECT_SETTERS);
        objectMapper.disable(MapperFeature.AUTO_DETECT_IS_GETTERS);
        objectMapper.disable(MapperFeature.USE_GETTERS_AS_SETTERS);
        objectMapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        objectMapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);
        objectMapper.disable(SerializationFeature.INDENT_OUTPUT);

        SimpleModule module = new SimpleModule();
        module.addSerializer(UUID.class, new UUIDSerializer());
        module.addDeserializer(UUID.class, new UUIDDeserializer());
        module.addSerializer(ItemStack.class, new ItemStackSerializer());
        module.addDeserializer(ItemStack.class, new ItemStackDeserializer());
        objectMapper.registerModule(module);

        objectWriter = objectMapper.writer();
        objectReader = objectMapper.reader();
    }

    public static void registerModule(Module module) {
        objectMapper.registerModule(module);

        objectWriter = objectMapper.writer();
        objectReader = objectMapper.reader();
    }

    public static void registerSerializer(Class clazz, JsonSerializer serializer) {
        SimpleModule module = new SimpleModule();
        module.addSerializer(clazz, serializer);
        registerModule(module);
    }

    public static void registerDeserializer(Class clazz, JsonDeserializer deserializer) {
        SimpleModule module = new SimpleModule();
        module.addDeserializer(clazz, deserializer);
        registerModule(module);
    }

    public static String serialize(Object obj) {
        try {
            return objectWriter.writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            LOGGER.error("Failed to serialize object: " + obj, e);
            return null;
        }
    }

    public static <T> T deserialize(String json, Class<T> clazz) {
        try {
            return objectReader.forType(clazz).readValue(json);
        } catch (JsonProcessingException e) {
            LOGGER.error("Failed to deserialize object: " + json, e);
            return null;
        }
    }
}