package com.io.yutian.elementoriginlib.tag;

import net.minecraft.nbt.*;
import net.minecraft.nbt.Tag;
import org.jetbrains.annotations.Nullable;
import org.json.JSONArray;
import org.json.JSONObject;

public class TagHelper {

    public static Object tagToJson(Tag tag) {
        switch (tag.getId()) {
            case 1: // ByteTag
                return ((ByteTag) tag).getAsByte();
            case 2: // ShortTag
                return ((ShortTag) tag).getAsShort();
            case 3: // IntTag
                return ((IntTag) tag).getAsInt();
            case 4: // LongTag
                return ((LongTag) tag).getAsLong();
            case 5: // FloatTag
                return ((FloatTag) tag).getAsFloat();
            case 6: // DoubleTag
                return ((DoubleTag) tag).getAsDouble();
            case 7: // ByteArrayTag
                return ((ByteArrayTag) tag).getAsByteArray();
            case 8: // StringTag
                return ((StringTag) tag).getAsString();
            case 9: // ListTag
                ListTag listTag = (ListTag) tag;
                JSONArray jsonArray = new JSONArray();
                for (Tag element : listTag) {
                    jsonArray.put(tagToJson(element));
                }
                return jsonArray;
            case 10: // CompoundTag
                CompoundTag compoundTag = (CompoundTag) tag;
                JSONObject jsonObject = new JSONObject();
                for (String key : compoundTag.getAllKeys()) {
                    jsonObject.put(key, tagToJson(compoundTag.get(key)));
                }
                return jsonObject;
            case 11:
                return ((IntArrayTag) tag).getAsIntArray();
            case 12:
                return ((LongArrayTag) tag).getAsLongArray();
            default:
                throw new IllegalArgumentException("Unsupported Tag type: " + tag.getId());
        }
    }

    public static Tag jsonToTag(Object json) {
        if (json instanceof JSONObject) {
            JSONObject jsonObject = (JSONObject) json;
            CompoundTag compoundTag = new CompoundTag();
            for (String key : jsonObject.keySet()) {
                compoundTag.put(key, jsonToTag(jsonObject.get(key)));
            }
            return compoundTag;
        } else if (json instanceof JSONArray) {
            JSONArray jsonArray = (JSONArray) json;
            ListTag listTag = new ListTag();
            for (int i = 0; i < jsonArray.length(); i++) {
                listTag.add(jsonToTag(jsonArray.get(i)));
            }
            return listTag;
        } else if (json instanceof String) {
            return StringTag.valueOf((String) json);
        } else if (json instanceof Integer) {
            return IntTag.valueOf((Integer) json);
        } else if (json instanceof Long) {
            return LongTag.valueOf((Long) json);
        } else if (json instanceof Double) {
            return DoubleTag.valueOf((Double) json);
        } else if (json instanceof Float) {
            return FloatTag.valueOf((Float) json);
        } else if (json instanceof Byte) {
            return ByteTag.valueOf((Byte) json);
        } else {
            throw new IllegalArgumentException("Unsupported JSON type: " + json.getClass());
        }
    }

    @Nullable
    public static ITag convert(Object object) {
        if (object instanceof Byte) {
            return new TagByte((byte) object);
        } else if (object instanceof Short) {
            return new TagShort((short) object);
        } else if (object instanceof Integer) {
            return new TagInt((int) object);
        } else if (object instanceof Long) {
            return new TagLong((long) object);
        } else if (object instanceof Float) {
            return new TagFloat((float) object);
        } else if (object instanceof Double) {
            return new TagDouble((double) object);
        } else if (object instanceof byte[]) {
            return new TagByteArray((byte[]) object);
        } else if (object instanceof String) {
            return new TagString((String) object);
        } else if (object instanceof int[]) {
            return new TagIntArray((int[]) object);
        } else if (object instanceof long[]) {
            return new TagLongArray((long[]) object);
        } else if (object instanceof Boolean) {
            return new TagByte((byte) ((boolean)object ? 1 : 0));
        }
        return null;
    }

    public static Object convertNMSNBT(ITag nbt) {
        return ITag.asNMS(nbt);
    }

}
